
#   oedit.tcl      Option editor

set frame OEDIT
set title "Experimental Option Editor"
set icontitle OEdit

set key "sys"
set Option ""
set value ""

set keys {}

proc update_Options {} {
   global env Current key

   set lines [get_Option_file]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace [K $lines [set lines {}]] [first $range] [expr [second $range] + 1]]
       set new 0
   } else {
       set new 1
   }

   lappend lines "## $key"
   foreach item [array names Current] {
       lappend lines "$item $Current($item)"
   }
   lappend lines "##"

   put_Option_file $lines
   
   if {$new == 1} start
}

proc delete_key {} {
   global key

   set lines [get_Option_file]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace [K $lines [set lines {}]] [first $range] [expr [second $range] + 1]]
   }

   put_Option_file $lines
   
   start
}

proc click_key {v} {
    global key
    set key $v
    :modify KEYT :value $key
    load_Options
}

proc click_Option {v} {
    global Current Option value
    
    set value $v
    set Current($Option) $value
    display_Options
}

proc click_delete_Option {} {
    global Current Option
    
    unset Current($Option)
    display_Options
}

proc load_Options {} {
    global env key Current
    
    catch {unset Current}

    set l {}
    foreach item [get_Options $key] {
        set Current([lindex $item 0]) [lindex $item 1]
    }
    display_Options
}

proc click_list {v} {
    global Option value
    
    set Option [first $v]
    set value [lreplace $v 0 0]
    
    :modify OPTION :value $Option
    :modify VALUE :value $value
}

proc display_Options {} {
   global Current

    set l {}
    foreach item [array names Current] {
        lappend l "[format "%30s %50s" $item $Current($item)]"
    }
    :modify LIST :append F :strings [lsort -dictionary -index 0 $l] :see 0
}

proc click arg {
   global debug Option
   if {$debug} then {insert-debug "click with argument \{$arg\}"}

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {

      KEYT -
      KEYB        {click_key $v}
      
      OPTION      {set Option $v}
      VALUE       {click_Option $v}
      
      LIST        {click_list $v}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc make-frame {} {
   global keys

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-name "[self icon-title]"
      
     :panel P1 :layout T
       :list LIST :rows 20  :cols 80 :exclusive T :scroll B
       
     :panel P2 :layout B :padx 10 :pady 10
      :layout H; :col-gap 20; :row-gap 5
       :text KEYT        :label Key  :w 16 
       :menubutton KEYB  :label "" :strings $keys
       :button SAVE      :label "Save Options"  :command update_Options
       :button DELETE    :label "Delete Option" :command click_delete_Option
#       :button DELKEY    :label "Delete Key"        :command delete_key
      :next-row T
       :text OPTION  :label "Option" :w 20 :shifters T
       :text VALUE       :label "Value"      :w 40 :shifters T

   :show T
}

proc start args {
   global started keys
   
   set keys {}
   foreach item [get_Option_file] {
       if {[string length $item] > 3 && [string compare -length 3 $item "## "] == 0} then {
           lappend keys [string range $item 3 end]
       }
   }
       
   make-frame
   set started 1
   if {$args != ""} then {click_key [first $args]} else {click_key "sys"}
}



proc get_Option_file {} {
    global env

#    look for the Option file 
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

proc put_Option_file {lines} {
   global env

#    look for the Option file 
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf w]} m]
    if {$z != 0} then {
        insert-log "Unable to access $pf - $m"
        return 1
    }

    foreach item $lines {
      if {$item != ""} {puts $f $item}
    }

    close $f
    return 0
}

proc get_Options {task} {
   global env

   set lines [get_Option_file]
   set key $task
   set range [find-keyword $lines $key]
   
   set Options {}

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           lappend Options [list [head $item] [lreplace $item 0 0]]
       }
   }
   return $Options
}
