
proc Set_V792_Options {} {
    global Options V792_Base

    load_Options

   if {![info exists Options(v792.channels)]} {set Options(v792.channels) 4096}
   if {![info exists Options(v792.offset)]} {set Options(v792.offset) 0}
   if {![info exists Options(v792.shift)]} {set Options(v792.shift) 0}
   if {![info exists Options(v792_GroupBase)]} {set Options(v792_GroupBase) $V792_Base}
   set V792_Base   $Options(v792_GroupBase)

    update_Options
}

proc Dump_Raw_VME_Data_V792 {data} {
     
     set info ""

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 24 & 7]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3f]
             set un [expr $word >> 13 & 1]
             set ov [expr $word >> 12 & 1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i; un= %3i; ov= %3i; data= 0x%5x" $word $event_type $channel $un $ov $adc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word >> 8 & 0x3f]
             append info "[format "0x%8x    %8s   module= %5i; channels= %6i" $word $event_type $geo $counter]"
            }
            4 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0xffffff]
             append info "[format "0x%8x    %8s   module= %5i; event data counter= %10i" $word $event_type $geo $counter]"
            }
            default {
             append info "$word - not valid datum\n"
            }
         }
         append info "\n"
     }
   return $info
}

proc define_V792_config {} {
   global VMEserver
    global V792_Base

   set maxmodule [inform vmecontrol inquire v792count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "792 0 [format "0x%x" $V792_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v792#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v792#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "792 $i [string range [first $m] 2 end]"
   }
}

proc create_V792_spectra {} {
    global SpectrumServer V792_Base
    global Options

   set maxmodule [inform vmecontrol inquire v792count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module v792#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v792#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v792.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum  v792#$i.adc$j"
          clear-footer
          error $m
      } else {
          set group [expr $V792_Base - 1 + $i]
          map_spectrum  "$spectrumname $group $j $Options(v792.offset) $Options(v792.shift)"
      }
    }
   }
}
