#    Diagnostic and test task for the NetVar service

set frame  NetVar
set title "NetVar Client"
set icon   NetVar

set started 0

package require ServerAccess 2.0    ;#  gets new version of EG command and Server Access procedures
package require NetVarClient

if {[info exists env(NetVarServer)] == 0} then {set MIDASAccessServer NetVar}  \
        else {set MIDASAccessServer $env(NetVarServer)}

set LogFlag 1

set DiagFileBase [file join $env(MIDASBASE) MIDAS_Acq tcl diag-scripts]

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim

set RoleList [list NetVar]
set NetVarServerList {}

set tchoice {float double int uint long ulong short ushort char uchar string}
set pchoice {rw ro}
set achoice {0 1}
set fchoice {x d}

set reg_name "*"
set reg_n ""
set reg_t [first $tchoice]
set reg_p [first $pchoice]
set reg_a [first $achoice]
set reg_f [first $fchoice]
set reg_i 0
set reg_data 0


proc make-serverlist {} {
  global NetVarServerList RoleList

#    find NetVar servers available

   set NetVarServerList ""
   if {[catch {eg "(experiment ?resources)"} m]} then {
       midas-warning "no experiment connected!!!"
   } else {
       foreach role $RoleList {
          set c [inform ex obtain_RPCServer_byRole $role]
          foreach item $c {
              append NetVarServerList " [first [inform ex obtain_RPCServerInfo $item $role]]"
          }

          set c [inform ex obtain_SOAPServer_byRole $role]
          foreach item $c {
              append NetVarServerList " [first [inform ex obtain_SOAPServerInfo $item $role]]"
          }
       }
   }
#    sort lists into sensible order
   set serverlist [lsort $NetVarServerList]
   if {[llength $serverlist] == 0} then {
      midas-warning "no NetVar servers found in current experiment!!!"
      set NetVarServerList {}
   } else {
      set NetVarServerList {}
      foreach server $serverlist {lappend NetVarServerList $server}
      :modify SERVERC :menu $NetVarServerList
   }
}


proc make-frame {} {
   global env
   global tchoice pchoice achoice fchoice

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20
    :text SERVERT         :label "NetVar Server" :w 12 
    :choice-stack SERVERC :label ""

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20
     :button LOAD :label "Load source file"  :command click_load
     :button REQ  :label "Show XML request"  :command click_request
     :button RESP :label "Show XML reply"    :command click_reply
     :hskip 50
     :button CLEAR :command ":modify LOG :empty T"

  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font variable
     configuretags

  :panel P3 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 10
      :text REG-NAME :label "Register Name:" :w 30 :value *
      :button REG-QUERY :label "Inquire Registers" :command inquire_registers
      :next-row T
      :message M5 :label "Attributes:"
      :col-gap 15
      :text REG-N     :label "variable name" :w 30
      :text REG-DATA  :label "data" :w 30
      :next-row T
      :choice-stack REG-T :label type :exclusive T :strings $tchoice :value 0
      :choice-stack REG-P :label permissions :exclusive T :strings $pchoice :value 0
      :choice-stack REG-F :label format :exclusive T :strings $fchoice :value 0
      :choice-stack REG-A :label array :exclusive T :strings $achoice :value 0
      :number REG-I :label index :min 0 :max 9999 :value 0

      :col-gap 10
      :next-row T
      :button REG-CR  :label "Create"           :command create_register
      :button REG-DL  :label "Delete"           :command delete_register
      :button REG-WA  :label "Write Attributes" :command write_attributes
      :button REG-RA  :label "Read Attributes"  :command read_attributes
      :button REG-RD  :label "Read"             :command read_register
      :button REG-WD  :label "Write"            :command write_register

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 50; :row-gap 10
     :button BAT :label "Execute file"
     :menubutton HISTORY   :label "History"  :strings [list ""]
     :button VERBOSE :label "Set Server Verbose" :command set_verbose
     :next-row T
     :colour local entry foreground black
     :text CMD  :label "command:" :w [expr $cols - 10]
    bind [:path CMD].entry <Meta-k>         "restorecommand last"
    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
    bind [:path CMD].entry <Up> "restorecommand previous"
    bind [:path CMD].entry <Down> "restorecommand next"

   open-frame

  :galley_size LOG $rows $cols

    focus [:path CMD].entry

   :show T
}


proc click-configure {W} {:galley_resize LOG}

proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {insert "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc click-bat args {
    global env MIDASAccessServer DiagFileBase

#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{Command Files}      {.bat}   }
      {{All Files}   *        }
    }
    set batfile [tk_getOpenFile -parent [self frame-name] -title "Diagnostic script Selection"  \
       -initialdir $DiagFileBase -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$batfile != ""} then {
       set directory [file dirname $batfile]
       set file [file tail $batfile]
       set z [midas-confirmation "Execute file $file for the VME card in VME crate $MIDASAccessServer"]
       if {$z == 0} then {return}
       uplevel #0 source $batfile
       set DiagFileBase $directory
    }
}


proc invoke {text} {
    global outputprompt inputprompt
    global history histmenu histlim nextcommand
    global AbortFlag LogFlag

    set AbortFlag 0

    set cmd ""
    foreach item [split $text ", "] {
        if {$item != ""} then {append cmd $item " "}
    }

    :modify LOG :usingtag taginput :strings "[set inputprompt]$cmd\n"

  if [info complete $cmd] {
       if {$cmd == "!!"} {
           set cmd [lindex $history end]
       } else {
           lappend history "$cmd"
           set history [lreplace [K $history [set history {}]] 0 0]
       }
  
    if {[first $cmd] == "repeat"} then {
        set count [second $cmd]
        set l [string length "repeat $count"]
        set cmd [string range $cmd [expr $l+1] end]
    } else {set count 1}

    if {$count == 0} then {set count 100000}
    for {set i 1} {$i <= $count} {incr i} {
       update
       if {$AbortFlag == 1} {:modify LOG :strings "Abort\n"; break}
       set result [catch {eval $cmd} msg]
       if {$result != 0} {
            :modify LOG :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
            if {$i <= $count} then {:modify LOG :usingtag tagoutput :strings "loop aborted due to error\n"}
            break;
       } else {
           if {$msg != ""} then {if {$LogFlag == 1 || $i == 1} then {:modify LOG :usingtag tagoutput :strings "$outputprompt$msg\n"}}
       }
    }
  }

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}


proc click_request {args} {
    global LastSOAPMethodName
    insert "[SOAP::dump -request $LastSOAPMethodName]"
}

proc click_reply {args} {
    global LastSOAPMethodName
    insert "[SOAP::dump -reply $LastSOAPMethodName]"
}

proc click_load {} {midas-browser R do_load}

proc do_load {args} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file

   set f [file join $directory $file]
   cd $directory
   insert "loading $f"
   set z [catch {source $f} m]
   if {$z != 0} then {insert "$m"}
}

proc do_command {W s} {
   global CurrentCommand LastMethodName
   set CurrentCommand "$s"
   set LastMethodName [first $CurrentCommand]
   if [info complete $s] {
       set z [catch {eval $s} m]
       if {$z != 0} then {insert "$m"} else {insert "$s=> $m"}
   }
   :modify CMD :value ""
}

proc restorecommand {args} {
    global CurrentCommand
    :modify CMD :value "$CurrentCommand"

}

proc insert {s} {:modify LOG :strings $s\n}

proc click arg {
   global history reg_name reg_n reg_t reg_p reg_a reg_f reg_i reg_data
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch $w  {

      SERVERT    {click-server $v}
      SERVERC    {click-server $i}

      REG-NAME   {set reg_name $v}
      REG-N      {set reg_n $v}
      REG-T      {set reg_t $i}
      REG-P      {set reg_p $i}
      REG-A      {set reg_a $i}
      REG-F      {set reg_f $i}
      REG-I      {set reg_i $v}
      REG-DATA   {set reg_data $v}

      CMD        {invoke "$v"}
      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}



proc create_register {args} {
   global reg_name reg_n reg_t reg_p reg_a reg_f reg_i

   set reply [MIDASDefineRegister $reg_name NetVar "-n $reg_n -t $reg_t -p $reg_p -a $reg_a -f $reg_f -i $reg_i"]

    if {[first $reply] == 0} {       ;#  good response
        insert "DefineRegister created $reg_name"
    } else {
        insert "DefineRegister returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc create_register {args} {
   global reg_name reg_n reg_t reg_p reg_a reg_f reg_i

   set reply [MIDASDefineRegister $reg_name NetVar "-n $reg_n -t $reg_t -p $reg_p -a $reg_a -f $reg_f -i $reg_i"]

    if {[first $reply] == 0} {       ;#  good response
        insert "DefineRegister created $reg_name"
    } else {
        insert "DefineRegister returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc inquire_registers {args} {
    global reg_name

    if {$reg_name == ""} then {
        set reply [MIDASInquireRegisters *]
    } else {
        set reply [MIDASInquireRegisters $reg_name]
    }

    if {[first $reply] == 0} {       ;#  good response
        if {[llength [second $reply]] == 0} then {
            insert "InquireRegisters found: no registers"
        } else {
            insert "InquireRegisters found:"
            insert "[second $reply]"
            set reg_name [first [second $reply]]
            :modify REG-NAME :value $reg_name
        }
    } else {
        insert "InquireRegisters returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc read_attributes {args} {
    global reg_name

    set reply [MIDASReadRegisterAttributes $reg_name]

    if {[first $reply] == 0} {       ;#  good response
        insert "ReadRegisterAttributes returned: [tail $reply]"
    } else {
        insert "ReadRegisterAttributes returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc read_register {args} {
    global reg_name reg_data

    set reply [MIDASReadRegister $reg_name]

    if {[first $reply] == 0} {       ;#  good response
        set reg_data "[tail $reply]"
        insert "ReadRegister returned $reg_data from $reg_name"
        :modify REG-DATA :value "$reg_data"
    } else {
        insert "ReadRegister returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc write_register {args} {
    global reg_name reg_data

    set reply [MIDASWriteRegister $reg_name $reg_data]

    if {[first $reply] == 0} {       ;#  good response
        insert "WriteRegister wrote $reg_data to $reg_name"
    } else {
        insert "WriteRegister returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc write_attributes {args} {
    global reg_name reg_n reg_t reg_p reg_a reg_f reg_i


    set attributes "-n $reg_n -t $reg_t -p $reg_p -a $reg_a -f $reg_f -i $reg_i"
    set reply [MIDASWriteRegisterAttributes $reg_name $attributes]

    if {[first $reply] == 0} {       ;#  good response
        insert "WriteRegisterAttributes wrote $attributes to $reg_name"
    } else {
        insert "WriteRegisterAttributes returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc set_verbose {args} {
    global reg_name reg_data

    set reply [MIDASSetVerbose $reg_name $reg_data]

    if {[first $reply] == 0} {       ;#  good response
        insert "MIDASSetVerbose wrote $reg_data to $reg_name"
    } else {
        insert "MIDASSetVerbose returned: rc=[first $reply] reason=[tail $reply]"
    }
}

proc delete_register {args} {
    global reg_name

    set reply [MIDASDeleteRegister $reg_name]

    if {[first $reply] == 0} {       ;#  good response
        insert "DeleteRegister deleted $reg_name"
    } else {
        insert "DeleteRegister returned: rc=[first $reply] reason=[tail $reply]"
    }
}



proc click-server {v} {
    global NetVarServerList MIDASAccessServer

    set  MIDASAccessServer $v
    :modify SERVERT :value $MIDASAccessServer
    :modify SERVERC :value [lsearch $NetVarServerList $MIDASAccessServer]

    set-footer "Connecting to server $MIDASAccessServer"
#    define-SERVER-registers

    clear-footer
    insert "Connected to server $MIDASAccessServer"

}

proc start args {
   global started NetVarServerList

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

#    built the choice menu for the server
   make-serverlist
   
   click-server [lindex $NetVarServerList 0]
}
